/* This is a test utility program for Winford Engineering's
 * SR24 serial I/O board, which was discussed in an article
 * in Nuts and Volts magazine.
 * 
 * This program uses the serial board API to program and 
 * control the I/O board.
 * 
 * Winford Engineering
 * www.winfordeng.com
 */
 
#include <stdio.h>
#include "ser_sr24.h"


/* define a structure to hold some information we need 
 * This structure is used to share information between
 * the main program and the event handler
 */
typedef struct
{
	ser_sr24 handle; /* handle of the serial board */
	int events_on;   /* nonzero if events should be displayed when they come in */
} prog_data;


/* NOTE NOTE NOTE NOTE:
 * If you are compiling on a Microsoft Windows platform, the event callback must
 * be defined as __stdcall calling convention.  The #ifdef below will do this
 * automatically if WIN32 has been defined.  WIN32 is defined by default in MSVC,
 * so this will probably work for a lot of people.
 * 
 * Just MAKE SURE that it is defined as __stdcall or there will be problems
 * with events.
 */

#ifdef WIN32
#define CALLCONVENTION __stdcall
#else
#define CALLCONVENTION
#endif

void CALLCONVENTION event_callback(int id, int value, void *extra)
{ /* This is the event callback function, which is called by
   * the serial API whenever a registered event occurs.
   * 
   * Because of the parameters passed when the event handler was set,
   * the "extra" parameter points to the prog_data structure that holds
   * variables we need.
   * 
   * This function actually does not do anything with events, unless
   * the user has enabled the display of events by choosing the item
   * off the menu.  In that case, the events_on member of the prog_data
   * structure will be nonzero.
   */
	prog_data *prgdata;

	prgdata = (prog_data*)extra;

	if(prgdata->events_on) /* If displaying of events is enabled */
		printf("\nEvent ID: %d\t New value: %d\n", id, value);
	/* otherwise ignore the event */
}


#define MAIN_MENU(p)(menu(p,\
               "-------------------------------------\n"\
		       "----   Main Menu                 ----\n"\
		       "-------------------------------------\n"\
		       "1 -- Open a serial I/O board\n"\
		       "2 @@ Setup / Configuration menu\n"\
		       "3 @@ Hardware event menu\n"\
		       "4 -- Verify connection to serial board\n"\
		       "5 -- Output to a digital port\n"\
		       "6 -- Output to a digital bit\n"\
		       "7 -- Input from a digital port\n"\
		       "8 -- Input from a digital bit\n"\
		       "9 -- Input from an analog channel\n"\
		       "10 - Close the serial board\n"\
		       "11 - Exit program\n"\
		       "-------------------------------------\n\n",\
		       1, 11))

#define CONFIG_MENU(p)(menu((p),\
               "-------------------------------------\n"\
               "---   Setup / Configuration Menu ----\n"\
               "-------------------------------------\n"\
               "1 - Reset the serial board\n"\
               "2 - Configure a port setting\n"\
               "3 - Display a port's settings\n"\
               "4 - Set timeout for inputs\n"\
               "5 - Back to Main Menu\n"\
               "-------------------------------------\n",\
               1, 5))

#define EVENT_MENU(p)(menu((p),\
               "-------------------------------------\n"\
               "---   Hardware Event Menu        ----\n"\
               "-------------------------------------\n"\
               "1 - Turn a byte event on or off\n"\
               "2 - Turn a bit event on or off\n"\
               "3 - Enable the displaying of events\n"\
               "4 - Wait for specific event(s)\n"\
               "5 - Back to Main Menu\n"\
               "-------------------------------------\n",\
               1, 5))

int getnumber(prog_data *prgdata, char *prompt); /* prompt for number until we get it.  returns then entered number */
int menu(prog_data *prgdata, char *menu, int lowlim, int highlim); /* prints the main menu of options and returns the choice picked */

void config_menu(prog_data *prgdata); /* handles the configuration menu */
void event_menu(prog_data *prgdata);

void openboard(prog_data *prgdata); /* opens the board. stores handle in data structure if successful */
void configport(prog_data *prgdata); /* configures a setting of a port */
void displayport(prog_data *prgdata); /* displays the current settings of a port */
void settimeout(prog_data *prgdata); /* sets the timeout for inputs from the board */
void verifyconn(prog_data *prgdata);
void outputport(prog_data *prgdata); /* Output a value to a port */
void outputbit(prog_data *prgdata);
void inputport(prog_data *prgdata);
void inputbit(prog_data *prgdata);
void inputanalog(prog_data *prgdata); /* Read an analog channel */
void byteevent(prog_data *prgdata); /* turn a byte event on or off */
void bitevent(prog_data *prgdata); /* turn a bit event on or off */
void displayevents(prog_data *prgdata); /* enable displaying of events */
void waitevents(prog_data *prgdata);  /* wait for a specific event(s) that the user specifies */

void waitenter(prog_data *prgdata, int prompt);
int handle_check(prog_data *prgdata);
void quit_program(prog_data *prgdata);


int main(int argc, char *argv[])
{
	prog_data data; /* declare structure to hold handle and more */
	prog_data *prgdata;

	/* initialize data structure */
	prgdata = &data;
	prgdata->handle = 0; /* board is not open */
	prgdata->events_on = 0; /* don't display events yet */
	
	while(1)
	{
		switch(MAIN_MENU(prgdata))
		{
		case 1: /* Open a serial board */
			if(prgdata->handle)
			{
				printf("\n\n**************************************\n"
				           "The serial board is already open.\n"
				           "**************************************\n\n");
				waitenter(prgdata, 1);
				break;
			}
			
			openboard(prgdata);
			break;
		case 2: /* Config menu */
			if(handle_check(prgdata) == 0)
				config_menu(prgdata);
			break;
		case 3: /* Event menu */
			if(handle_check(prgdata) == 0)
				event_menu(prgdata);
			break;
		case 4: /* Verify connection */
			if(handle_check(prgdata) == 0)
				verifyconn(prgdata);
			break;
		case 5: /* Output to a port */
			if(handle_check(prgdata) == 0)
				outputport(prgdata);
			break;
		case 6: /* Output to a bit */
			if(handle_check(prgdata) == 0)
				outputbit(prgdata);
			break;
		case 7: /* Input from a port */
			if(handle_check(prgdata) == 0)
				inputport(prgdata);
			break;
		case 8: /* Input from a bit */
			if(handle_check(prgdata) == 0)
				inputbit(prgdata);
			break;
		case 9: /* Input from analog */
			if(handle_check(prgdata) == 0)
				inputanalog(prgdata);
			break;
		case 10: /* Close board */
			if(prgdata->handle == 0)
				printf("\nBoard is not open\n\n");
			else
			{
				ser_sr24_close(prgdata->handle);
				prgdata->handle = 0; /* shows that we've closed the board */
				printf("\nSerial board has been closed\n\n");
			}
			waitenter(prgdata, 1);
			break;
		case 11: /* Exit program */
			quit_program(prgdata);
			break;
		default: /* error */
		}
	}
	
	return(0);
}


void config_menu(prog_data *prgdata)
{
/* This function handles the interface of the configuration menu
 */
	int result;

	while(1)
	{
		switch(CONFIG_MENU(prgdata))
		{
		case 1: /* Reset the serial board */
			if( (result=ser_sr24_reset_board(prgdata->handle)) )
				printf("\nERROR resetting board: %s\n\n", ser_sr24_error_string(result));
			else
				printf("\nBoard reset successfully.\n\n");
	
			waitenter(prgdata, 1);
				
			break;
		case 2: /* Configure a port setting */
			configport(prgdata);
			break;
		case 3: /* Display a port's settings */
			displayport(prgdata);
			break;
		case 4: /* Set timeout for inputs */
			settimeout(prgdata);
			break;
		case 5: /* Back to Main Menu */
			return;
			break;
		}
	}
}

void event_menu(prog_data *prgdata)
{ /* Handles the interface of the event menu */

	while(1)
	{
		switch(EVENT_MENU(prgdata))
		{
		case 1: /* Turn a byte event on or off */
			byteevent(prgdata);
				
			break;
		case 2: /* Turn a bit event on or off */
			bitevent(prgdata);
			
			break;
		case 3: /* Enable the displaying of events */
			displayevents(prgdata);
			
			break;
		case 4: /* Wait for specific event(s) */
			waitevents(prgdata);
			
			break;
		case 5: /* Back to Main Menu */
			return;
			break;
		}
	}

}

void openboard(prog_data *prgdata)
{
/* This function allows the user to open a serial I/O board.
 * 
 * It returns the handle on success, or zero on failure.
 */
	ser_sr24 handle;
	ser_event_spec evspec;
	int baud;
	unsigned int baudconst;
	char buf[300];
	char serial[300];
	int result;

	printf("\nWhat serial port is the board is attached to?\n"
	       "On Microsoft Windows, enter something like COM1 or COM2\n"
	       "On Linux, enter something like /dev/ttyS0 or /dev/ttyS1\n\n"
	       "Enter serial port: ");

	if(fgets(buf, sizeof(buf), stdin) == 0)
		quit_program(prgdata);

	sscanf(buf, "%s", serial); /* read the string, leave the newline */
	
	/* get the baud rate now */
	printf("\n\nWhat baud rate is the serial board set to use?\n"
	       "Valid choices are 2400, 4800, 9600, or 19200.\n");

	while(1)
	{
		baud = getnumber(prgdata, "Enter baud rate: ");

		switch(baud)
		{
		case 2400:
			baudconst = SER_2400;
			break;
		case 4800:
			baudconst = SER_4800;
			break;
		case 9600:
			baudconst = SER_9600;
			break;
		case 19200:
			baudconst = SER_19200;
			break;
		default:
			printf("Invalid baud rate\n");
			continue;
		}
		break;
	}

	handle = ser_sr24_open(serial, baudconst, &result);
	if(handle == 0)
		printf("\nThis error occurred:\n%s\n\n", ser_sr24_error_string(result));
	else
	{
		/* Now, we'll use a callback event handler for any events the user 
		 * may choose to register
		 */
	
		evspec.type = 1;
		evspec.eventfn = event_callback; /* set callback to function we defined above */
		evspec.extra = prgdata; /* give callback the address of our data structure */
	
		if( (result=ser_sr24_set_event_handler(handle, &evspec)) )
			printf("\nERROR setting up event handler: %s\n\n", ser_sr24_error_string(result));
		else
			printf("\nSerial board successfully opened.\n\n");
	
		prgdata->handle = handle;
	}

	waitenter(prgdata, 1);
	
}

void waitevents(prog_data *prgdata)
{  /* wait for a specific event(s) that the user specifies */
	int events[50]; /* Actually, we should only need 24 spots max */
	unsigned int numevents=0;
	int ev; /* temporary variable to hold the entered event number */
	int i;
	int result;
	int value;
	int timeout;
	char buf[100];
	
	printf("\nWhich event ID's would you like to wait for?\n"
	         "Enter each event ID on a separate line, entering\n"
	         "a zero when you are finished\n"
	         "OR, enter a zero on the first line to wait for ANY event\n");

	do
	{
		ev = getnumber(prgdata, "Enter next event ID or 0 if you're done: ");
		if(ev < 0)
			printf("\nEvent ID's must be positive\n");
		else
		{
			events[numevents] = ev; /* store event in the array */
			numevents++; /* keep track of how many events we have */
		}
	}while( (ev) && numevents < (sizeof(events) / sizeof(events[0])) ); /* keep going until we get a zero, or until the user fills up our array */

	numevents--; /* don't count the last zero */
	/* NOTE: numevents could possibly be zero now if the user entered
	 * a 0 value right away.  In this case, the API will return any
	 * event that occurs.
	 */
	
	do
	{
		printf("\nHow long should we wait for events before timing out?\n");
		do
		{
			timeout = getnumber(prgdata, "Enter timeout in milliseconds (0 for infinite): ");
		}while(timeout < 0);

		printf("\nNow waiting for these events: ");
		if(numevents)
		{
			for(i=0; i<numevents; i++)
				printf("%d ", events[i]);
		}
		else
			printf("ANY");

		printf("\n\nWaiting for events...\n");

		result = ser_sr24_wait_event(prgdata->handle, events, numevents, timeout, &value);

		if(result < 0)
			printf("\nERROR waiting for events: %s\n\n", ser_sr24_error_string(result));
		else
			printf("\nEvent %d occurred.  New value: %d\n\n", result, value);

		printf("Would you like to wait for these same events again? (y/n): ");
		
		if(fgets(buf, sizeof(buf), stdin) == 0)
			quit_program(prgdata);

	}while(buf[0] == 'Y' || buf[0] == 'y');
}	


void displayevents(prog_data *prgdata)
{ /* This function sets a flag in the data structure allowing the 
   * event callback to print out events as they come in
   */

	printf("\nThe display of events is about to be enabled.\n"
	       "After you hit ENTER, any events that come in\n"
	       "will be displayed on the screen as they come in.\n"
	       "When you are done viewing events, hit ENTER\n"
	       "again to return to the menu.\n");
	waitenter(prgdata, 1);
	prgdata->events_on=1;
	printf("\nEvents will now be displayed when they occur.\n"
	       "Hit enter when you're ready to exit this mode\n");
	
	waitenter(prgdata, 0); /* don't prompt as it would be confusing */
	prgdata->events_on=0;

	printf("\nEvent display has been turned off.\n");
	waitenter(prgdata, 1);
}

void byteevent(prog_data *prgdata)
{ /* turn a byte event on or off */
	int result;
	int id;
	int port;

	printf("\nEnable/Disable byte event on which port?\n");
	do
	{
		port = getnumber(prgdata, "Enter port number (1-3): ");
	}while(port < 1 || port > 3);

	printf("Enter a positive ID for this event, or zero to turn it off.\n");
	id = getnumber(prgdata, "ID: ");
	/* If they enter an invalid ID, let the API catch the error */

	if( (result=ser_sr24_register_event(prgdata->handle, port, -1, id)) )
		printf("\nERROR registering event: %s\n\n", ser_sr24_error_string(result));
	else
	{
		if(id)
			printf("\nByte event successfully enabled\n");
		else /* If user entered zero, event has been disabled */
			printf("\nByte event successfully disabled\n");
	}
	
	waitenter(prgdata, 1);
}

void bitevent(prog_data *prgdata)
{ /* turn a bit event on or off */
	int result;
	int id;
	int port;
	int bit;

	printf("\nEnable/Disable bit event on which port?\n");
	do
	{
		port = getnumber(prgdata, "Enter port number (1-3): ");
	}while(port < 1 || port > 3);

	printf("\nWhich bit of the port?\n");
	do
	{
		bit = getnumber(prgdata, "Enter bit number (0-7): ");
	}while(bit < 0 || bit > 7);

	printf("Enter a positive ID for this event, or zero to turn it off.\n");
	id = getnumber(prgdata, "ID: ");
	/* If they enter an invalid ID, let the API catch the error */

	if( (result=ser_sr24_register_event(prgdata->handle, port, bit, id)) )
		printf("\nERROR registering event: %s\n\n", ser_sr24_error_string(result));
	else
	{
		if(id)
			printf("\nBit event successfully enabled\n");
		else /* If user entered zero, event has been disabled */
			printf("\nBit event successfully disabled\n");
	}
	
	waitenter(prgdata, 1);
}


void inputanalog(prog_data *prgdata)
{
	/* Read a value from an analog channel */
	int result;
	int value;
	int channel;

	printf("\nInput from which analog channel?\n");
	do
	{
		channel = getnumber(prgdata, "Enter channel number (0-7): ");
	}while(channel < 0 || channel > 7);

	if( (result=ser_sr24_input_analog(prgdata->handle, channel, &value)) )
		printf("\nERROR reading channel: %s\n\n", ser_sr24_error_string(result));
	else
		printf("\nChannel value = %d\n", value);

	waitenter(prgdata, 1);
}

void inputport(prog_data *prgdata)
{
	int result;
	int value;
	int port;

	printf("\nInput from which port?\n");
	do
	{
		port = getnumber(prgdata, "Enter port number (1-3): ");
	}while(port < 1 || port > 3);	

	if( (result=ser_sr24_input_byte(prgdata->handle, port, &value)) )
		printf("\nERROR reading value: %s\n\n", ser_sr24_error_string(result));
	else
		printf("\nPort value = %d\n", value);

	waitenter(prgdata, 1);
}

void inputbit(prog_data *prgdata)
{
	int result;
	int value;
	int port;
	int bit;

	printf("\nInput from which port?\n");
	do
	{
		port = getnumber(prgdata, "Enter port number (1-3): ");
	}while(port < 1 || port > 3);	

	printf("\nInput from which bit on the port?\n");
	do
	{
		bit = getnumber(prgdata, "Enter bit number (0-7): ");
	}while(bit < 0 || bit > 7);
	
	if( (result=ser_sr24_input_bit(prgdata->handle, port, bit, &value)) )
		printf("\nERROR reading value: %s\n\n", ser_sr24_error_string(result));
	else
	{
		if(value)
			printf("\nBit is ON\n");
		else
			printf("\nBit is OFF\n");
	}

	waitenter(prgdata, 1);
}


void outputport(prog_data *prgdata)
{ /* Output a value to a digital port.
   * User can specify port and value
   */
	int result;
	int port;
	int value;

	printf("\nOutput to which port?\n");
	do
	{
		port = getnumber(prgdata, "Enter port number (1-3): ");
	}while(port < 1 || port > 3);	

	printf("\nOutput which value?\n");
	do
	{
		value = getnumber(prgdata, "Enter value (0-255): ");
	}while(value < 0 || value > 255);	

	if( (result=ser_sr24_output_byte(prgdata->handle, port, value)) )
		printf("\nERROR writing value: %s\n\n", ser_sr24_error_string(result));
	else
		printf("\nValue written successfully.\n\n");

	waitenter(prgdata, 1);
}

void outputbit(prog_data *prgdata)
{ /* Outputs a bit to a port the user chooses */
	int result;
	int port;
	int bit;
	int value;

	printf("\nOutput to which port?\n");
	do
	{
		port = getnumber(prgdata, "Enter port number (1-3): ");
	}while(port < 1 || port > 3);	

	printf("\nOutput to which bit on the port?\n");
	do
	{
		bit = getnumber(prgdata, "Enter bit number (0-7): ");
	}while(bit < 0 || bit > 7);
	
	printf("\nTurn the bit on or off?\n");
	value = getnumber(prgdata, "0 - Turn bit off\n"
	                           "nonzero - Turn bit on\n"
	                           "Enter value: ");

	if( (result=ser_sr24_output_bit(prgdata->handle, port, bit, value)) )
		printf("\nERROR writing value: %s\n\n", ser_sr24_error_string(result));
	else
		printf("\nValue written successfully.\n\n");

	waitenter(prgdata, 1);
}

void verifyconn(prog_data *prgdata)
{ /* verify the serial board connection */
	int result;

	if( (result=ser_sr24_verify_connection(prgdata->handle)) )
		printf("\nERROR verifying connection: %s\n\n", ser_sr24_error_string(result));
	else
		printf("\nConnection is good.\n\n");

	waitenter(prgdata, 1);
}
		
void settimeout(prog_data *prgdata)
{
	int result;
	int timeout;

	printf("\nHow long should input functions wait for a response\n"
	         "before timing out?  Initial default is one second.\n");

	do
	{
		timeout = getnumber(prgdata, "Enter timeout in milliseconds (0 for infinite): ");
	}while(timeout < 0);

	if( (result=ser_sr24_set_timeout(prgdata->handle, (unsigned int)timeout)) )
		printf("\nERROR setting timeout: %s\n", ser_sr24_error_string(result));
	else
		printf("\nTimeout set successfully.\n\n");

	waitenter(prgdata, 1);
}
	
void displayport(prog_data *prgdata)
{
/* displays the current settings of a port chosen by the user */
	int port;
	int mode;

	printf("\nDisplay the properties of which port? (1-3)\n");

	do
	{
		port = getnumber(prgdata, "Enter port number: ");
	}while(port < 1 || port > 3);

	printf("\n\nPort %d:\n", port);

	mode = ser_sr24_get_portmode(prgdata->handle, port, SR24_BUFFER);
	if(mode == SR24_BUFFER_ENABLED)
		printf("Buffer: Enabled\n");
	else if(mode == SR24_BUFFER_DISABLED)
		printf("Buffer: Disabled\n");

	mode = ser_sr24_get_portmode(prgdata->handle, port, SR24_DIRECTION);
	if(mode == SR24_DIRECTION_INPUT)
		printf("Direction: Input\n");
	else if(mode == SR24_DIRECTION_OUTPUT)
		printf("Direction: Output\n");
	
	mode = ser_sr24_get_portmode(prgdata->handle, port, SR24_TYPE);
	if(mode == SR24_TYPE_DIGITAL)
		printf("Type: Digital\n");
	else if(mode == SR24_TYPE_ANALOG)
		printf("Type: Analog\n");

	waitenter(prgdata, 1);
}

void configport(prog_data *prgdata)
{ /* configures the setting of a port based on the user's request */
	int result;
	int port;
	int property;
	int mode;

	printf("\nWhich port do you want to configure? (1-3)\n");

	do
	{
		port = getnumber(prgdata, "Enter port number: ");
	}while(port < 1 || port > 3);

	printf("\nWhich property do you want to set?\n");

	do
	{
		property = getnumber(prgdata, "Valid properties:\n"
		                              "1 - Tri-State Buffer: Enabled / Disabled\n"
		                              "2 - Port direction: Input / Output\n"
		                              "3 - Port type: Digital / Analog\n"
		                              "Enter property number: ");
	}while(property < 1 || property > 3);

	switch(property)
	{
	case 1: /* buffer */
		printf("\nEnable or Disable the buffer?\n");
		do
		{
			mode = getnumber(prgdata, "\n"
			                          "1 - Buffer enabled\n"
			                          "2 - Buffer disabled\n"
			                          "Enter choice: ");
		}while(mode < 1 || mode > 2);

		/* set up the arguments for the function.  Function is called a little later */
		property = SR24_BUFFER;
		if(mode == 1)
			mode = SR24_BUFFER_ENABLED;
		else
			mode = SR24_BUFFER_DISABLED;

		break;
	case 2: /* direction */
		printf("\nPut port into Input or Output mode?\n");
		do
		{
			mode = getnumber(prgdata, "\n"
			                          "1 - Input mode\n"
			                          "2 - Output mode\n"
			                          "Enter choice: ");
		}while(mode < 1 || mode > 2);

		property = SR24_DIRECTION;
		if(mode == 1)
			mode = SR24_DIRECTION_INPUT;
		else
			mode = SR24_DIRECTION_OUTPUT;
			
		break;

	case 3: /* analog / digital */
		printf("\nPut port into Digital or Analog mode?\n");
		do
		{
			mode = getnumber(prgdata, "\n"
			                          "1 - Digital mode\n"
			                          "2 - Analog mode\n"
			                          "Enter choice: ");
		}while(mode < 1 || mode > 2);

		property = SR24_TYPE;
		if(mode == 1)
			mode = SR24_TYPE_DIGITAL;
		else
			mode = SR24_TYPE_ANALOG;
			
		break;
	}

	if( (result=ser_sr24_set_portmode(prgdata->handle, port, property, mode)) )
		printf("\nERROR setting portmode: %s\n", ser_sr24_error_string(result));
	else
		printf("\nPort mode set successfully.\n\n");

	waitenter(prgdata, 1);

}


void quit_program(prog_data *prgdata)
{
	/* This function is called when the user wants to quit the program */

	printf("\n\n"); /* our final farewell */
	
	if(prgdata->handle == 0)
		ser_sr24_close(prgdata->handle);
		
	exit(0);
}

int getnumber(prog_data *prgdata, char *prompt)
{
/* Prompt user for number until we get one.  
 * 
 * If it encounters an error in the input stream, like
 * EOF, it calls the quit_program function, which ends the
 * whole program.
 */

	char buf[100];
	int num;

	while(1)
	{
		printf("%s", prompt);
		
		if(fgets(buf, sizeof(buf), stdin) == 0)
			quit_program(prgdata); /* probably EOF */

		if(sscanf(buf, "%d\n", &num) <= 0)
		{
			printf("\nPlease enter a number.\n");
			continue;
		}
		else
			break;
	}

	return(num);
}

void waitenter(prog_data *prgdata, int prompt)
{
/* Wait for user to hit enter.  If there's an error,
 * such as EOF, quit the program.
 * 
 * If prompt is nonzero, the function prompts for the user
 * to hit enter, otherwise no prompt is given.
 */
	char buf[100];

	if(prompt)
		printf("Hit Enter to continue...\n");
		
	if(fgets(buf, sizeof(buf), stdin) == 0)
		quit_program(prgdata);
}
	
int handle_check(prog_data *prgdata)
{
	/* returns zero on OK, nonzero if not open */
	
	if(prgdata->handle == 0)
	{
		printf("\n**************************************\n"
		         "You must first open a serial board    \n"
		         "**************************************\n\n");
		waitenter(prgdata, 1);
		
		return(-1);
	}
	else
		return(0);
}

int menu(prog_data *prgdata, char *menu, int lowlim, int highlim)
{
/* Prints the menu of options passed in by "menu", 
 * and returns the number of the choice picked.
 * 
 * NOTE: menu should have its own newline at the end.
 * 
 * This function checks to ensure that the user enters a valid number
 * between the lowlim and highlim specified.
 * 
 * Be sure that the menu doesn't have negative options, because
 * a negative return code from this function is an error.
 * 
 * Returns: number of the choice picked, or
 *          a negative number if there was a failure,
 *          such as EOF.
 */
	int choice;

	while(1)
	{
		printf("%s", menu);

		choice = getnumber(prgdata, "Enter choice: ");
		
		if(choice < lowlim || choice > highlim)
		{
			printf("\nInvalid menu choice.  Valid choices are between %d and %d\n", lowlim, highlim);
			waitenter(prgdata, 1);
			continue;
		}
		else
			break;
	}	

	return(choice);
}
